package com.hp.user.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.toolkit.SqlHelper;
import com.hp.user.client.entity.*;
import com.hp.user.client.service.AccountService;
import com.hp.user.client.service.MessageService;
import com.hp.user.service.constants.UserConstants;
import com.hp.user.service.dao.AccountMapper;
import com.hp.user.service.entity.Account;
import com.hp.user.service.entity.CustomerCompany;
import com.hp.user.service.entity.ElectricianGroup;
import com.hp.user.service.redis.RedisOperation;
import com.hp.user.service.utils.WeixinSignUtil;

import redis.clients.jedis.Jedis;

import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * <p>
 * 用户信息表 服务实现类
 * </p>
 *
 * @author 王怡静
 * @since 2020-11-20
 */
@Service
public class AccountServiceImpl implements AccountService {
	
    @Autowired
    private MessageService messageService;
    
    @Autowired
    private HttpServletRequest request;

    @Autowired
    private AccountMapper accountMapper;

    @Override
    public Result login(String phone, String verificationCode) {
    	Result result = new Result();
        try {
            //需要判断phone是否存在
            DataResult<Account> checkPhoneResult = checkPhone(phone);
            Account account = checkPhoneResult.getData();
            HttpSession session = request.getSession();
            if(null != account) {
            	String code = messageService.getVerificationCode(phone);
                if (StringUtils.equals(verificationCode, code)) {
                    //登陆成功
                	messageService.deleteVerificationCode(phone);
                	String sessionId = session.getId();
                	account.setWxOpenId(null);
                	Jedis jedis = RedisOperation.getRedis();
          		  	String key = UserConstants.SESSION_PRIFX+sessionId;
          		  	jedis.setex(key, UserConstants.EXPIRE_SECONDS, JSONObject.toJSONString(account));
                } else {
                    //登陆失败
                    //返回错误信息
                	result.setCode("-1");
                	result.setMessage("验证码错误！");
                }
            }else {
            	result.setCode("-1");
            	result.setMessage("手机号码未注册！");
            }
        } catch (Exception e) {
        	e.printStackTrace();
        	result.setCode("-1");
        	result.setMessage("系统出现问题");
        }
        return result;
    }

    @Override
    public DataResult<Account> checkPhone(String phone) {
    	DataResult<Account> result = new DataResult<Account>();
        //不存在为true
        try {

            QueryWrapper<Account> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("phone", phone);
            queryWrapper.eq("sys_deleted", 0);
            Account account = accountMapper.selectOne(queryWrapper);
            if (null != account) {
                //表示手机以及存在，需要返回错误
                result.setCode("-1");
            	result.setMessage("手机号已经存在");
            	result.setData(account);
            }
        } catch (Exception e) {
        	result.setCode("-1");
        	result.setMessage("系统出现问题");
        }
        return result;

    }

    @Override
    public void register(AccountVO accountVo) {
        try {
        	Result result = checkPhone(accountVo.getPhone());
            if (null == result.getMessage()) {
                Account account = new Account();
                BeanUtils.copyProperties(accountVo, account);
                Long id = IdWorker.getId();
                account.setUserId(id.toString());
                accountMapper.insert(account);
            }
        } catch (Exception e) {
        		
        }

    }

    @Override
    public void bindWx(String userIdStr, String wxNumber) {
        try {
			Account account = new Account();
			account.setUserId(userIdStr);
			account.setWxNumber(wxNumber);
			account.setWxNumber(wxNumber);
			accountMapper.updateById(account);
		}catch(Exception e) {
			
		}
		
	}

	@Override
	public DataResult<AccountVO> wxAuthorization(AccountVO account,String wxCode) {
		DataResult<AccountVO> result = new DataResult<AccountVO>();
		JSONObject data = null;
		try {
	        data = WeixinSignUtil.getCode2Session(wxCode);
	        String errmsg = data.getString("errmsg");
	        if(null != errmsg) {
	        	result.setCode("-1");
	        	result.setCode(errmsg);
	        	return result;
	        }
	        String openid = data.getString("openid");
	        QueryWrapper<Account> queryWrapper = new QueryWrapper<>();
			queryWrapper.eq("wx_open_id", openid);
			Account findAccount = accountMapper.selectOne(queryWrapper);
			
			if(null == findAccount) {
				account = new AccountVO();
				String session = data.getString("session_key");
				account.setSession(session);
				account.setWxOpenId(openid);
				result.setCode("6000");
	        	result.setMessage("微信需要绑定");
	        	result.setData(account);
			}else {
				if(null != account) {
					//表示需要更新微信信息
					findAccount.setWxName(account.getWxName());
					findAccount.setWxPortrait(account.getWxPortrait());
					accountMapper.updateById(findAccount);
				}
				BeanUtils.copyProperties(findAccount, account);
				result.setData(account);
			}
	        
		}catch(Exception e) {
			result.setCode("-1");
        	result.setMessage("系统出现问题");
		}
		return result;
		
	}

	@Override
	public DataResult<AccountVO> wxRegister(AccountVO accountVO, String verificationCode) {
		DataResult<AccountVO> result = new DataResult<AccountVO>();
		try {
			if(!StringUtils.isBlank(accountVO.getPhone())) {
				Result checkPhone = checkPhone(accountVO.getPhone());
				if(null == checkPhone.getMessage()) {
//					if(StringUtils.isBlank(accountVO.getWxNumber())) {
//						result.setCode("-1");
//			        	result.setMessage("微信号不能为空！");
//						return result;
//					}
					String openid = accountVO.getWxOpenId();
					if(StringUtils.isBlank(openid)) {
						result.setCode("-1");
			        	result.setMessage("微信的openId不能为空！");
						return result;
					}
					 QueryWrapper<Account> queryWrapper = new QueryWrapper<>();
					queryWrapper.eq("wx_open_id", openid);
					Integer count = accountMapper.selectCount(queryWrapper);
					if(count > 0) {
						result.setCode("-1");
			        	result.setMessage("微信的opeanId已经绑定过！");
						return result;
					}
					String code = messageService.getVerificationCode(accountVO.getPhone());
					if(StringUtils.equals(verificationCode, code)) {
						messageService.deleteVerificationCode(accountVO.getPhone());
						Account account = new Account();
						BeanUtils.copyProperties(accountVO, account);
						Long id = IdWorker.getId();
						account.setUserId(id.toString());
						accountMapper.insert(account);
						BeanUtils.copyProperties(account, accountVO);
						result.setData(accountVO);
					}else {
						result.setCode("-1");
			        	result.setMessage("验证码不正确！");
						return result;
					}
					
				}else {
					result.setCode("-1");
		        	result.setMessage(checkPhone.getMessage());
					return result;
				}
			}else {
				result.setCode("-1");
	        	result.setMessage("手机号不能为空！");
				return result;
			}
			
		}catch(Exception e) {
			e.printStackTrace();
			result.setCode("-1");
        	result.setMessage("系统内部错误！");
		}
		return result;
	}

    @Override
    public Result insert(AccountVO account) {
        Account temp = new Account();
        int flag = 0;
        Result result = new Result();
        try {
            BeanUtils.copyProperties(account, temp);
            flag = accountMapper.insert(temp);
        } catch (Exception e) {

        }
        if (SqlHelper.retBool(flag)) {
            result.setCode("0");
        } else {
            result.setCode("-1");
            result.setMessage("添加失败");
        }
        return result;
    }

    @Override
    public DataResult<AccountVO> queryByUserId(String userId) {
        AccountVO accountVO = new AccountVO();
        DataResult<AccountVO> result = new DataResult<>();
        try{
//            Account temp = accountMapper.selectById(userId);
//            QueryWrapper<> queryWrapper = new QueryWrapper();
            Account account = accountMapper.queryUserInfoAndCompanyInfo(userId);
            // 这个已经被删除
            if (account == null){
                result.setCode("-1");
                result.setMessage("账号不存在/已删除");
                return result;
            }
            else {
            	account.setWxOpenId(null);
                BeanUtils.copyProperties(account, accountVO);
                result.setData(accountVO);
            }
        }catch(Exception e){
        	e.printStackTrace();
        	result.setCode("-1");
            result.setMessage("系统错误");
        }
        return result;
    }

    @Override
    public Result delete(String userId) {
        Account temp;
        int flag = 0;
        Result result = new Result();
        try {
            temp = accountMapper.selectById(userId);
            if (temp == null) {
                result.setCode("-1");
                result.setMessage("未查找到此账号");
                return result;
            }
            temp.setSysDeleted((short) 1);
            flag = accountMapper.updateById(temp);
        } catch (Exception e) {

        }
        if (SqlHelper.retBool(flag)) {
            result.setCode("0");
        } else {
            result.setCode("-1");
            result.setMessage("删除失败");
        }
        return result;
    }

    @Override
    public Result update(AccountVO account) {
        Result result = new Result();
        Account temp = new Account();
        int flag = 0;
        try {
            BeanUtils.copyProperties(account, temp);
            flag = accountMapper.updateById(temp);
        } catch (Exception e) {

        }
        if (SqlHelper.retBool(flag)) {
            result.setCode("0");
        } else {
            result.setCode("-1");
            result.setMessage("更新失败");
        }
        return result;
    }

    @Override
    public PageResult<AccountVO> queryAccountsPageByCompanyIdAndUserNameAndPhoneAndStatus(String companyId, String userName, String phone, Integer status, Long current, Integer size) {
        PageResult<AccountVO> result = new PageResult<>();
        IPage<Account> page = new Page<>(current, size);
        QueryWrapper<Account> wrapper = new QueryWrapper<>();
        try {
            if (companyId != null) {
                wrapper.eq("company_id", companyId);
            }
            wrapper.orderByDesc("sys_create_time");
            if (userName != null) wrapper.like("user_name", userName);
            if (phone != null) wrapper.like("phone", phone);
            if (status == 0 || status == 1) wrapper.eq("status", status);
            accountMapper.selectPage(page, wrapper);
            List<Account> accounts = page.getRecords();
            List<AccountVO> accountVOs = new ArrayList<>();
            AccountVO temp;
            for (Account account : accounts) {
                temp = new AccountVO();
                BeanUtils.copyProperties(account, temp);
                accountVOs.add(temp);
            }
            result.setCode("0");
            result.setDataList(accountVOs);
            result.setTotal(page.getTotal());
            result.setCurrent(current);
            result.setSize(size);
        } catch (Exception e) {
        	e.printStackTrace();
        }
        return result;
    }

    @Override
    public Result batchChangeStatusOfUser(String[] userIds, Short status) {
        Account account;
        Result result = new Result();
        int flag = 0;
        try {
            for (String userId : userIds) {
                account = accountMapper.selectById(userId);
                if (account == null) {
                    result.setCode("-1");
                    result.setMessage("账号Id为 " + account.getUserId() + " 的账号不存在");
                    return result;
                }
                account.setStatus(status);
                flag = accountMapper.updateById(account);
            }
        } catch (Exception e) {

        }
        if (SqlHelper.retBool(flag)) {
            result.setCode("0");
        } else {
        	
            result.setCode("-1");
            result.setMessage("更新失败");
        }
        return result;
    }

    @Override
    public DataResult<AccountVO> queryAccountByPhone(String phone) {
        DataResult<AccountVO> result = new DataResult<>();
        QueryWrapper<Account> wrapper = new QueryWrapper<>();
        try {
            wrapper.eq("phone", phone);
            Account account = accountMapper.selectOne(wrapper);
            if (account == null) {
                result.setCode("-1");
                result.setMessage("未找到此手机号对应的用户");
            } else {
                AccountVO temp = new AccountVO();
                BeanUtils.copyProperties(account, temp);
                result.setCode("0");
                result.setData(temp);
            }
        } catch (Exception e) {
        	e.printStackTrace();
        	result.setCode("-1");
            result.setMessage("更新失败");
        }
        return result;
    }

	@Override
	public DataResult<AccountVO> queryUserBySession() {
		DataResult<AccountVO> result = new DataResult<AccountVO>();
		try {
			HttpSession session = request.getSession();
			String sessionId = session.getId();
			
			String value = null;
			AccountVO accountVO = null;
			if(!StringUtils.isBlank(sessionId)) {
				Jedis jedis = RedisOperation.getRedis();
				String key = UserConstants.SESSION_PRIFX+sessionId;
				value = jedis.get(key);
				accountVO = JSONObject.parseObject(value, AccountVO.class);
				
			}
			result.setData(accountVO);
//			String userId = ;
//			session.
		}catch(Exception e) {
			e.printStackTrace();
        	result.setCode("-1");
        	result.setMessage("系统出现问题");
		}
		return result;
	}
}
