package com.hp.user.service.impl;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.common.utils.CollectionUtils;
import org.apache.ibatis.session.RowBounds;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.hp.user.client.entity.DataResult;
import com.hp.user.client.entity.ElectricianVO;
import com.hp.user.client.entity.FuzzyQuery;
import com.hp.user.client.entity.ListResult;
import com.hp.user.client.entity.PageQuery;
import com.hp.user.client.entity.PageResult;
import com.hp.user.client.entity.Result;
import com.hp.user.client.service.ElectricianService;
import com.hp.user.service.dao.CertificateMapper;
import com.hp.user.service.dao.ElectricianMapper;
import com.hp.user.service.entity.Electrician;

@Service
public class ElectricianServiceImpl implements ElectricianService{
	
	@Autowired
	private ElectricianMapper electricianMapper;
	
	@Autowired
	private CertificateMapper certificateMapper;

	@Override
	@Transactional
	public Result createElectrician(ElectricianVO electricianVO) {
		Result result = new Result();
		try {
			String phone = electricianVO.getPhone();
			String name = electricianVO.getName();
			String serviceProviderId = electricianVO.getServiceProviderId();
			if(StringUtils.isBlank(phone)) {
				result.setCode("-1");
				result.setMessage("手机号不能为空！");
				return result;
			}
			if(StringUtils.isBlank(name)) {
				result.setCode("-1");
				result.setMessage("姓名能为空！");
				return result;
			}
			if(StringUtils.isBlank(serviceProviderId)) {
				result.setCode("-1");
				result.setMessage("服务商ID不能为空！");
				return result;
			}
			Electrician electrician = new Electrician();
			BeanUtils.copyProperties(electricianVO, electrician);
			String electricianId = IdWorker.getIdStr();
			electrician.setElectricianId(electricianId);
			electricianMapper.insert(electrician);
		}catch(Exception e) {
			result.setCode("-1");
			result.setMessage("系统内部错误");
			throw e;
		}
		return result;
	}

	@Override
	public PageResult<ElectricianVO> queryElectrician(PageQuery query) {
		PageResult<ElectricianVO> result = new PageResult<ElectricianVO>();
		try {
			Integer page = query.getPage();
			Integer count = query.getSize();
			if(page == null || page == 0) {
				return result;
			}
			QueryWrapper<Electrician> queryWrapper = null;
			List<FuzzyQuery> fuzzyList = query.getFuzzyList();
			if(CollectionUtils.isNotEmpty(fuzzyList)) {
				queryWrapper = new QueryWrapper<Electrician>();
				for(FuzzyQuery<?> fuzzy : fuzzyList) {
					queryWrapper.like(fuzzy.getColumn(), fuzzy.getValue());
				}
			}
			List<Electrician> list = null;
			Long total = null;
			if(page == -1) {
				list = electricianMapper.selectList(queryWrapper);
			}else {
				Page<Electrician> electricianPage = electricianMapper.selectPage(new Page<>(page,count), queryWrapper);
				list = electricianPage.getRecords();
				total = electricianPage.getTotal();
			}
			List<ElectricianVO> newList = new ArrayList<>();
			if(CollectionUtils.isNotEmpty(list)) {
				ElectricianVO electricianVO;
				for(Electrician electrician : list) {
					electricianVO = new ElectricianVO();
					BeanUtils.copyProperties(electrician, electricianVO);
					newList.add(electricianVO);
				}
			}
			result.setTotal(total);
			result.setDataList(newList);
		}catch(Exception e) {
			result.setCode("-1");
			result.setMessage("系统内部错误");
		}
		return result;
	}

	@Override
	public Result updateElectrician(ElectricianVO electricianVO) {
		Result result = new Result();
		try {
			Electrician electrician = new Electrician(electricianVO.getElectricianId(),electricianVO.getName(),
					electricianVO.getDictionariesId(),electricianVO.getCertificateUrl(),electricianVO.getRemarks(),electricianVO.getStatus());
			electricianMapper.updateById(electrician);
		}catch(Exception e) {
			e.printStackTrace();
			result.setCode("-1");
			result.setMessage("系统内部错误");
		}
		return result;
	}

	@Override
	public DataResult<ElectricianVO> queryElectricianById(String electricianId) {
		DataResult<ElectricianVO> result = new DataResult<ElectricianVO>();
		try {
			Electrician electrician = electricianMapper.selectById(electricianId);
			ElectricianVO electricianVO = null;
			if(null != electrician) {
				electricianVO = new ElectricianVO();
				BeanUtils.copyProperties(electrician, electricianVO);
			}
			result.setData(electricianVO);
		}catch(Exception e) {
			result.setCode("-1");
			result.setMessage("系统内部错误");
		}
		return result;
	}

	@Override
	public Result updateStatusBatch(String[] electricianIds,Integer status) {
		Result result = new Result();
		try {
			UpdateWrapper<Electrician> updateWrapper = new UpdateWrapper<Electrician>();
			updateWrapper.in("electrician_id", electricianIds);
			Electrician electrician = new Electrician();
			electrician.setStatus(status);
			electricianMapper.update(electrician, updateWrapper);
		}catch(Exception e) {
			result.setCode("-1");
			result.setMessage("系统内部错误");
		}
		return result;
	}

	@Override
	public Result deleteElectricianById(String electricianId) {
		Result result = new Result();
		try {
			UpdateWrapper<Electrician> updateWrapper = new UpdateWrapper<Electrician>();
			updateWrapper.eq("electrician_id", electricianId);
			Electrician electrician = new Electrician();
			electrician.setSysDeleted(1);
			electricianMapper.update(electrician, updateWrapper);
		}catch(Exception e) {
			result.setCode("-1");
			result.setMessage("系统内部错误");
		}
		return result;
	}
	
	

}
