package com.hp.user.service.impl;

import java.util.Map;
import java.util.Random;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.hp.user.client.entity.Result;
import com.hp.user.client.service.MessageService;
import com.hp.user.service.constants.UserConstants;
import com.hp.user.service.redis.RedisOperation;
import com.hp.user.service.utils.SendSmsUtil;

import redis.clients.jedis.Jedis;

/**
 * 信息服务实现类
 * @author yeqid
 * @since 2020/11/26
 *
 */
@Service
public class MessageServiceImpl implements MessageService {
	
	@Autowired
	private SendSmsUtil sendSmsUtil;	
	

	@Override
	public Result sendVerificationCode(String phone) {
		Result result = new Result();
		try {
			Random rd = new Random();
			
			StringBuilder code = new StringBuilder();
			for(int i=0;i<6;i++) {
				code.append(rd.nextInt(10));
			}
			String codeStr = code.toString();
			Map<?, ?> map = sendSmsUtil.sendVerificationCode(phone, codeStr);
			if("OK".equals(map.get("Code"))) {
				//调用阿里云接口成功
				//调用成功后需要将code缓存到redis中
				Jedis jedis = RedisOperation.getRedis();
				jedis.setex(UserConstants.VERIFICATION_CODE+phone, UserConstants.EXPIRE_SECONDS, codeStr);
				jedis.close();
			}else {
				System.out.println(map);
				result.setCode("-1");
				result.setMessage("短信发送失败");
			}
		}catch(Exception e) {
			e.printStackTrace();
			result.setCode("-1");
			result.setMessage("系统内部错误");
		}
		return result;
	}

	@Override
	public String getVerificationCode(String phone) {
		Jedis jedis = RedisOperation.getRedis();
		String key = UserConstants.VERIFICATION_CODE+phone;
		String code = jedis.get(key);
//		jedis.del(key);
		return code;
	}
	
	public void deleteVerificationCode(String phone) {
		Jedis jedis = RedisOperation.getRedis();
		String key = UserConstants.VERIFICATION_CODE+phone;
		jedis.del(key);
	}

}
